function [Xoptimum,Objective,Constraint_Violation,NFEs,FMIN,Elapsed_Time]=NNA_Const_Ver1(objective_function,constraints,LB,UB,nvars,npop,max_it)

% Last Revised: 12th August 2018

% Constrained Version 1 "Neural Network Algorithm" (NNA) Using Feasible Approach (Deb 2001)

% This code is prepared for single objective function (minimization), constrained, and continuous optimization problems.
% Note that in order to obey the copy-right rules, please cite our published paper properly. Thank you.

% Ali Sadollah, Hassan Sayyaadi, Anupam Yadav (2018) A dynamic metaheuristic optimization model inspired by biological nervous systems: Neural network algorithm, Applied Soft Computing, 71, pp. 747-782.

% INPUTS:

% objective_function:           Objective function which you wish to minimize or maximize
% constraints:                  Imposed constrained
% LB:                           Lower bound of a problem
% UB:                           Upper bound of a problem
% nvars:                        Number of design variables
% npop                          Population size
% max_it:                       Maximum number of iterations

% OUTPUTS:

% Xoptimum:                     Global optimum solution
% Objective:                    Cost of global optimum solution
% Constraint_Violation:         Violation of imposed constraints
% NFEs:                         Number of function evaluations
% FMIN:                         Cost reduction history
% Elapsed_Time:                 Elasped time for solving an optimization problem

%% ========================================================================
% Default values for the NNA
format long g
if (nargin <6 || isempty(npop)), npop=50; end
if (nargin <7 || isempty(max_it)), max_it=1000; end
%==========================================================================
%% --------------------Initialization--------------------------------------
X_LB=repmat(LB,npop,1);
X_UB=repmat(UB,npop,1);

beta=1;
epss=1e-06;

ind.position=[];
ind.cost=[];
ind.const=[];

pop=repmat(ind,npop,1);

%------------------ Creating Random Initial Population --------------------
for i=1:npop
    pop(i).position=LB+((UB-LB).*rand(1,nvars));
    pop(i).cost=objective_function(pop(i).position);
    c=constraints(pop(i).position);
    pop(i).const=sum(c(c>epss));
end
%----------------Sort for selecting the Target solution -------------------
X_Minus=[];
aa=[pop.const];
COST_MINUS=[pop(aa<=epss).cost];
if ~isempty(COST_MINUS)
    X_Minus=pop(aa<=epss);
    [~,INDEX_M]=sort(COST_MINUS);
    X_Minus=X_Minus(INDEX_M);
end

X_PLUS=[];
SUM_C_PLUS=aa(aa>epss);
if ~isempty(SUM_C_PLUS)
    X_PLUS=pop(aa>epss);
    [~,INDEX_P]=sort(SUM_C_PLUS);
    X_PLUS=X_PLUS(INDEX_P);
end

pop=[X_Minus;X_PLUS];

XTarget=pop(1);   % Best obtained solution so called Target Solution

% ------------- Initializing random weight matrix with constraint of Summation of each column = 1
ww=ones(1,npop)*0.5;
w=diag(ww);

for i=1:npop
    t=rand(1,npop-1)*0.5;
    t=(t./sum(t))*0.5;
    w(w(:,i)==0,i)=t;
end

%--------------------------------------------------------------------------
% w=rand(npop,npop);       %% An alternative way of initializing weights
% for i=1:npop
%     w(:,i)=w(:,i)./sum(w(:,i));       % Summation of each column = 1
% end

wtarget=w(:,1);     % Selected Target Weight 
%% -------------------- Main Loop for the NNA -----------------------------
FMIN=zeros(max_it,1);
XP=zeros(npop,nvars);
tic
x_pattern=pop;
for ii=1:max_it
    
    for jj=1:npop
        XP(jj,:)=x_pattern(jj).position;
    end
    %------------------ Creating new solutions ----------------------------
    x_new=w*XP;
    XP=x_new+XP;
    %------------------- Updating the weights -----------------------------
    for i=1:npop
        w(:,i)=abs(w(:,i)+((wtarget-w(:,i))*2.*rand(npop,1)));
    end
    
    for i=1:npop
        w(:,i)=w(:,i)./sum(w(:,i));    % Summation of each column = 1
    end
    %----------------------- Creat new input solutions --------------------
    for i=1:npop
        
        if rand<beta
            
            %------------- Bias for input solutions -----------------------
            N_Rotate=ceil(beta*nvars);
            
            xx=LB+(UB-LB).*rand(1,nvars);
            rotate_postion=randperm(nvars);rotate_postion=rotate_postion(1:N_Rotate);
            
            for m=1:N_Rotate
                XP(i,rotate_postion(m))=xx(m);
            end
            %---------- Bias for weights ----------------------------------
            N_wRotate=ceil(beta*npop);
            
            w_new=rand(N_wRotate,npop);
            rotate_position=randperm(npop);rotate_position=rotate_position(1:N_wRotate);
            
            for j=1:N_wRotate
                w(rotate_position(j),:)=w_new(j,:);
            end
            
            for iii=1:npop
                w(:,iii)=w(:,iii)./sum(w(:,iii));  % Summation of each column = 1
            end
            
        else
            %------------ Transfer Function Operator ----------------------
            XP(i,:)=XP(i,:)+(XTarget.position-XP(i,:))*2.*rand(1,nvars);
        end
    end
    % ---------------------- Bias Reduction -------------------------------
    beta=beta*0.99;
    if beta<0.01
        beta=0.05;
    end
    
    % beta=1-((1/max_it)*ii);        % An alternative way of reducing the value of beta
    %----------------------------------------------------------------------
    XP=max(XP,X_LB);    XP=min(XP,X_UB);
    
    for jj=1:npop
        x_pattern(jj).position=XP(jj,:);
    end
    %-------------- Calculating objective function values -----------------
    for i=1:npop
        x_pattern(i).cost=objective_function(x_pattern(i).position);
        c=constraints(x_pattern(i).position);
        x_pattern(i).const=sum(c(c>epss));
    end
    %% ------ Selection ---------------------------------------------------
    POP_New=[pop;x_pattern];
    
    X_Minus=[];    
    aa=[POP_New.const];
    COST_MINUS=[POP_New(aa<=epss).cost];
    if ~isempty(COST_MINUS)
        X_Minus=POP_New(aa<=epss);
        [~,INDEX_M]=sort(COST_MINUS);
        X_Minus=X_Minus(INDEX_M);
    end
    
    X_PLUS=[];
    SUM_C_PLUS=aa(aa>epss);
    if ~isempty(SUM_C_PLUS)
        X_PLUS=POP_New(aa>epss);
        [~,INDEX_P]=sort(SUM_C_PLUS);
        X_PLUS=X_PLUS(INDEX_P);
    end
    wtarget=w(:,1);
    POP_New=[X_Minus;X_PLUS];
    
    x_pattern=POP_New(1:npop);
    pop=POP_New(1:npop);

    XTarget=POP_New(1);   % Best obtained solution so called Target Solution
    %% Display
    disp(['Iteration: ',num2str(ii),'   Fmin= ',num2str(XTarget.cost),'  Sum_Const= ',num2str(XTarget.const),'   beta= ',num2str(beta)]);

    FMIN(ii)=XTarget.cost;
    
end
%% -------------------------------- NNA Finishes --------------------------
plot(FMIN,'Linewidth',3);
xlabel('Number of Iterations','FontSize',24,'FontWeight','bold');
ylabel('Objective Function Value','FontSize',24,'FontWeight','bold');
Xoptimum=XTarget.position;
Objective=objective_function(Xoptimum);  % Best obtained optimum solution
c=constraints(Xoptimum);
Constraint_Violation=sum(c(c>epss));
Elapsed_Time=toc;
NFEs=npop*max_it;
end